from ape_ethereum.transactions import TransactionType

from ape_polygon.ecosystem import PolygonConfig


def test_gas_limit(polygon):
    assert polygon.config.local.gas_limit == "max"


def test_default_transaction_type(polygon):
    assert polygon.config.mainnet.default_transaction_type == TransactionType.DYNAMIC


def test_mainnet_fork_not_configured():
    obj = PolygonConfig.model_validate({})
    assert obj.mainnet_fork.required_confirmations == 0


def test_mainnet_fork_configured():
    data = {"mainnet_fork": {"required_confirmations": 555}}
    obj = PolygonConfig.model_validate(data)
    assert obj.mainnet_fork.required_confirmations == 555


def test_custom_network():
    data = {"apenet": {"required_confirmations": 333}}
    obj = PolygonConfig.model_validate(data)
    assert obj.apenet.required_confirmations == 333



from typing import ClassVar, cast

from ape_ethereum.ecosystem import (
    BaseEthereumConfig,
    Ethereum,
    NetworkConfig,
    create_network_config,
)

NETWORKS = {
    # chain_id, network_id
    "mainnet": (137, 137),
    "amoy": (80002, 80002),
}


class PolygonConfig(BaseEthereumConfig):
    NETWORKS: ClassVar[dict[str, tuple[int, int]]] = NETWORKS
    mainnet: NetworkConfig = create_network_config(block_time=2, required_confirmations=1)
    amoy: NetworkConfig = create_network_config(block_time=2, required_confirmations=1)


class Polygon(Ethereum):
    fee_token_symbol: str = "POL"

    @property
    def config(self) -> PolygonConfig:  # type: ignore[override]
        return cast(PolygonConfig, self.config_manager.get_config("polygon"))



from ape import plugins


@plugins.register(plugins.Config)
def config_class():
    from ape_polygon.ecosystem import PolygonConfig

    return PolygonConfig


@plugins.register(plugins.EcosystemPlugin)
def ecosystems():
    from ape_polygon.ecosystem import Polygon

    yield Polygon


@plugins.register(plugins.NetworkPlugin)
def networks():
    from ape.api.networks import (
        LOCAL_NETWORK_NAME,
        ForkedNetworkAPI,
        NetworkAPI,
        create_network_type,
    )

    from ape_polygon.ecosystem import NETWORKS

    for network_name, network_params in NETWORKS.items():
        yield "polygon", network_name, create_network_type(*network_params)
        yield "polygon", f"{network_name}-fork", ForkedNetworkAPI

    # NOTE: This works for development providers, as they get chain_id from themselves
    yield "polygon", LOCAL_NETWORK_NAME, NetworkAPI


@plugins.register(plugins.ProviderPlugin)
def providers():
    from ape.api.networks import LOCAL_NETWORK_NAME
    from ape_node import Node
    from ape_test import LocalProvider

    from ape_polygon.ecosystem import NETWORKS

    for network_name in NETWORKS:
        yield "polygon", network_name, Node

    yield "polygon", LOCAL_NETWORK_NAME, LocalProvider


def __getattr__(name: str):
    if name == "NETWORKS":
        from .ecosystem import NETWORKS

        return NETWORKS

    elif name == "Polygon":
        from .ecosystem import Polygon

        return Polygon

    elif name == "PolygonConfig":
        from .ecosystem import PolygonConfig

        return PolygonConfig

    else:
        raise AttributeError(name)


__all__ = [
    "NETWORKS",
    "Polygon",
    "PolygonConfig",
]
